#include "contextmenu_item_matroska.h"

static bool getSaveFileName(pfc::string_base & p_out, const pfc::string_base & p_filename) {
	const int MAXPATH = 1024;
    wchar_t pathname[MAXPATH];
    OPENFILENAME ofn;

    ZeroMemory(&ofn,sizeof(ofn));
	lstrcpy((wchar_t*)pathname, pfc::stringcvt::string_wide_from_utf8(p_filename).get_ptr());

    ofn.lStructSize = sizeof(ofn);
    ofn.lpstrFilter = L"All files (*.*)\0\0";
    ofn.nFilterIndex = 1;
	ofn.lpstrDefExt = L"";
	ofn.lpstrFile = (wchar_t*)pathname;
    ofn.nMaxFile = MAXPATH;
    ofn.lpstrTitle = L"Save as";
    ofn.Flags=OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_OVERWRITEPROMPT;

	if(GetSaveFileName(&ofn)){
		p_out = pfc::stringcvt::string_utf8_from_wide(pathname, sizeof(pathname));
		return true;
	} else {
		return false;
    }
}

bool contextmenu_item_node_leaf_matroska::saveFile(const matroska::attachment & p_attachment, const pfc::string_base & p_path)
{
	service_ptr_t<file> write_file;
	abort_callback_impl _abort;
	filesystem::g_open_write_new(write_file, p_path, _abort);
    pfc::array_t<unsigned char> buffer;
	t_size remain = p_attachment.get_size();
    t_size position = static_cast<t_size>(p_attachment.get_position());
    const int BUFFER_SIZE = 102400;
	while (remain > 0) {
        buffer.set_size(BUFFER_SIZE);
        t_size size = buffer.get_size();
		if (remain < size) {
			size = remain;
			buffer.set_size(size);
		}
        p_attachment.get(static_cast<void *>(buffer.get_ptr()), position, size);
		write_file->write(static_cast<void *>(buffer.get_ptr()), size, _abort);
		remain -= size;
        position += size;
	}
	return true;
}

void contextmenu_item_node_leaf_matroska::execute(const pfc::list_base_const_t<metadb_handle_ptr> & p_data,const GUID & p_caller)
{
	pfc::string8 path;
    matroska::attachment _attachment = m_owner->getOwner()->getAttachmentList()->get_item(m_index);
	pfc::string8 filename = m_owner->getFileName(m_index);
	filename.fix_filename_chars();
	switch (m_menuindex) {
	case Open:
		uGetTempPath(path);
		path.add_string(filename);
        if (saveFile(_attachment, path)) {
			uShellExecute(0, "open", path, 0, 0, SW_SHOW);
		}
		break;
	case SaveAs:
		if (getSaveFileName(path, filename)) {
			saveFile(_attachment, path);
		}
		break;
	}
}

bool contextmenu_item_node_root_popup_matroska_list::get_display_data(pfc::string_base & p_out,unsigned & p_displayflags,const pfc::list_base_const_t<metadb_handle_ptr> & p_data,const GUID & p_caller)
{
	pfc::string8 name = getFileName(m_index);
	if (!name.is_empty()) {
		name.add_string(" (");
        pfc::string8 mime_type;
        m_owner->getAttachmentList()->get_item(m_index).get_mime_type(mime_type);
        name += mime_type;
		name.add_string(")");
	}
	p_out = name;
	return true;
}

pfc::string8 contextmenu_item_node_root_popup_matroska_list::getFileName(t_size p_index)
{
    pfc::string8 name;
	m_owner->getAttachmentList()->get_item(p_index).get_name(name);
    return name;
}

bool contextmenu_item_node_root_popup_matroska::get_display_data(pfc::string_base & p_out,unsigned & p_displayflags,const pfc::list_base_const_t<metadb_handle_ptr> & p_data,const GUID & p_caller)
{
	m_owner->get_item_name(0, p_out);
	return true;
}

t_size contextmenu_item_node_root_popup_matroska::get_children_count()
{
	return m_owner->getAttachmentList()->get_count();
}